@@ -1,22 +1,22 @@
-# Copyright (C) 2017 Markus Baaske. All Rights Reserved.
-# This code is published under the L-GPL.
+# Copyright (C) 2018 Markus Baaske. All Rights Reserved.
+# This code is published under the GPL (>=3).
 #
 # File: 	krige.R
-# Date:  	22/10/2017
+# Date:  	14/03/2018
 # Author: 	Markus Baaske
 #
-# Define kriging prediction, variance interpolation,
-# quasi-deviance and Mahalanobis distance for use of
-# a simulated version of GMM
+# Implements kriging prediction, variance approximation methods,
+# quasi-deviance and Mahalanobis distance function as a criterion
+# for QL estimation
 
 #' @name estim
 #'
-#' @title Kriging prediction and estimation of derivatives
+#' @title Kriging prediction and numerical approximation of derivatives
 #'
 #' @description 	
 #'
 #' @param models 	object of class \code{krige} either as a list of covariance models or
-#' 	 				class `\code{covModel}` as a single covariance model, see \code{\link{setCovModel}}
+#' 	 				class \code{covModel} as a single covariance model, see \code{\link{setCovModel}}
 #' @param points 	matrix or list of points to predict the sample means of statistics
 #' @param Xs		matrix of sample points
 #' @param data		data frame of sample means of statistics at sampled points
@@ -27,13 +27,12 @@
 #' 		 variances if `\code{krig.type}` equals to "\code{var}")}
 #'  \item{jacobian}{ list of Jacobians at predicted values of sample means of statistics}  
 #'
-#' @details The function can be used to predict any values by kriging given a covariance model. In particular, we use it to predict
-#'  the sample mean of any statistic. Each covariance model is given as an element of the list `\code{models}` including its own trend
-#'  model and covariance function name. There are two types of kriging predictors available. First, the \emph{dual kriging} predictor,
-#'  set by `\code{krig.type}`="\code{dual}" or the one based on the calculation of prediction variances, if `\code{krig.type}` equals
-#'  "\code{var}". Both result in exactly the same predicted values and only differ by whether or not kriging variances are calculated.
-#'  The measurements (data), e.g. sample means for each statistic, must be given as column vectors where each row corresponds to a
-#'  sample point in the data frame `\code{data}`. 
+#' @details The function can be used to predict any values by kriging given a covariance model. Each covariance model is given as an
+#'  element of the list `\code{models}` including its own trend model and covariance function name. There are two types of kriging predictors
+#'  available. First, the \emph{dual kriging} predictor, set by `\code{krig.type}`="\code{dual}" or the one based on the calculation of prediction
+#'  variances, setting `\code{krig.type}` to "\code{var}". Both types result in exactly the same predicted values and only differ by whether or not
+#'  kriging variances should be computed. The data, e. g. sample means of each statistic, must be given as column vectors where each row corresponds
+#'  to a sample point in the data frame given in the argument `\code{data}`. 
 #'
 #' @examples 
 #' data(normal) 
@@ -93,7 +92,7 @@ estim.covModel <- function(models, points, Xs, data,
 #'
 #' @details
 #'   The function `\code{jacobian}` computes the partial derivatives of sample means of the statistics
-#'   as columns and for each component of the parameter vector as rows by forward differences.
+#'   as columns and for each component of the parameter vector as rows by forward difference approximations.
 #' 
 #' @rdname estim
 #' 
@@ -136,10 +135,9 @@ jacobian.covModel <- function(models, points, Xs, data,
 #'  } 
 #' 
 #' @details For a list of fitted covariance models the function \emph{predictKM} predicts the
-#' 	 sample means of statistics at (unsampled) points, calculates the prediction
-#' 	 variances (if `\code{krig.type}` equals "\code{var}") at these points and extracts the results.
-#' 	 Note that, since we aim on predicting the simulation "error free" value of the sample means,
-#'   we use a \emph{smoothing} kriging predictor (see [2, Sec. 3.7.1]).
+#' 	 sample means of statistics at (unsampled) points, calculates the prediction variances, if applicable,
+#'   at these points and extracts the results. Note that, since we aim on predicting the "error free" value of the sample means,
+#'   we use the \emph{smoothing} kriging predictor as described in [2, Sec. 3.7.1].
 #'
 #' @param models   list of covariance models, see \code{\link{setCovModel}}
 #' @param ... 	   further arguments passed to function \code{\link{estim}}
@@ -192,7 +190,7 @@ varKM  <- function(models, ...) {
 #'  The function \emph{extract} either returns the predicted values, the
 #'  prediction variances or the kriging weights for each point. 
 #'
-#' @return \item{\code{extract}}{matrix of corresponding values (see details)}  
+#' @return \item{\code{extract}}{ matrix of corresponding values (see details)}  
 #'
 #' @rdname krige
 #' @export
@@ -202,7 +200,18 @@ extract <- function(X, type = c("mean","sigma2","weights")) UseMethod("extract",
 #' @export
 extract.krigResult <- function(X, type=c("mean","sigma2","weights")) {
 	type <- match.arg(type)
-	do.call(rbind,lapply(X,"[[",type))
+	switch(type,
+	   "weights" = { 
+		   if(!is.null(attr(X,"weights")))		# only for dual kriging weights!
+			 return( t(do.call(rbind,(attr(X,"weights")))) )
+		   lapply(X,function(x) {
+			 w <- matrix(unlist(x$weights),nrow=length(x$weights[[1]]))
+			 colnames(w) <- names(x$weights)
+			 w
+		 	})	
+	   },
+	   do.call(rbind,lapply(X,"[[",type))
+	)
 }
 
 # intern
@@ -228,41 +237,42 @@ varLOGdecomp <- function(L) {
 #' @description Approximating the variance-covariance matrix of statistics
 #'
 #' @param qsd		object of class \code{\link{QLmodel}}
-#' @param W			weight matrix for weighted average approximation of variance matrix
-#' @param theta	    parameter vector for weighted average approximation of variance matrix 
+#' @param W			weight matrix for weighted average approximation
+#' @param theta	    parameter vector for weighted average approximation 
 #' @param cvm		list of fitted cross-validation models, see \code{\link{prefitCV}}
 #' @param useVar 	logical, if \code{TRUE}, then use prediction variances (see details)
-#' @param doInvert	if \code{TRUE}, return the inverse of the approximated variance matrix
+#' @param doInvert	if \code{TRUE}, return the inverse of the variance matrix approximation
 #'
 #' @return
 #' 	List of variance matrices with the following structure:
-#' 	\item{VTX}{ Variance matrix approximation}
+#' 	\item{VTX}{ variance matrix approximation}
 #'  \item{sig2}{ if applicable, kriging prediction variances of statistics at `\code{theta}`}
-#'  \item{var}{ Matrix `\code{VTX}` with added variances `\code{sig2}` as diagonal terms}
+#'  \item{var}{ matrix `\code{VTX}` with added variances `\code{sig2}` to the diagonal terms}
 #'  \item{inv}{ if applicable, the inverse of either `\code{VTX}` or `\code{var}`}
 #'
-#' @details	The function estimates the variance matrix of statistics at some (unsampled) point by either
+#' @details	The function estimates the variance matrix of statistics at some (unsampled) point `\code{theta}` by either
 #'  averaging (the \emph{Cholesky} decomposed terms or matrix logarithms) over all simulated variance matrices
 #'  of statistics at previously evaluated points of the parameter space or by a kriging approach which treats the Cholesky
-#'  decomposed terms of each variance matrix as the data vector for kriging.
+#'  decomposed terms of each variance matrix as a data vector for kriging.
 #' 
 #'  In addition, a Nadaraya-Watson kernel-weighted average approximation can also be applied in order to bias the variance
 #'  estimation towards a more locally weighted estimation, where smaller weights are assigned to points being more
-#'  distant to an estimate of the unknown model parameter `\code{theta}`. A reasonable symmetric weighting matrix 
+#'  distant to an estimate of the model parameter `\code{theta}`. A reasonable symmetric weighting matrix 
 #'  `\code{W}` of size equal to the problem dimension, say \code{q}, can be freely chosen by the user. In addition, the user can select
 #'  different types of variance averaging methods such as "\code{cholMean}", "\code{wcholMean}", "\code{logMean}", "\code{wlogMean}"
 #'  or "\code{kriging}" defined by `\code{qsd$var.type}`, where the prefix "\code{w}" indicats its corresponding weighted version of
-#'  approximation. Depending on the type of kriging for the statistics, `\code{qsd$krig.type}`, prediction variances
-#'  \eqn{\sigma(\theta)} of the sample mean of statistics at point `\code{theta}` are added or not. If `\code{qsd$krig.type}` equals
+#'  the approximation type. Depending on the type of kriging for the statistics, `\code{qsd$krig.type}`, prediction variances
+#'  \eqn{\sigma(\theta)} of the sample mean of statistics at `\code{theta}` are added. If `\code{qsd$krig.type}` equals
 #'  "\code{dual}", see \code{\link{QLmodel}}, then no prediction variances are used at all and thus the variance matrix estimate of
 #'  the statistics only includes the variances due to simulation replications and not the ones due to the use of kriging approximations
 #'  of the statistics. Otherwise, including the prediction variances, the mean variance matrix estimate is given by
 #'  \deqn{ \hat{V}+\textrm{diag}(\sigma(\theta)),} 
 #' 	where \eqn{\hat{V}} denotes one of the above variance approximation types.
+#' 
 #'  The prediction variances \eqn{\sigma} are either derived from the kriging results of statistics or based on a (possibly more robust)
-#'  CV approach (see vignette). Finally, we can switch off using prediction variances of either type by setting `\code{useVar}`=\code{FALSE}.
-#'  In general, this should be avoided. However, if the estimation problem under investigation is \emph{simple enough},
-#'  then this choice may be still appropriate.
+#'  cross-validation (CV) approach, see vignette for details. Finally, we can switch off using prediction variances of either type by setting
+#'  `\code{useVar}`=\code{FALSE}. In general, this should be avoided. However, if the estimation problem under investigation is \emph{simple enough},
+#'  then this choice may be still useful.
 #'    
 #' @examples 
 #'  data(normal)
@@ -349,7 +359,8 @@ covarTx <- function(qsd, W = NULL, theta = NULL, cvm = NULL, useVar = FALSE, doI
 			if(var.type=="cholMean" || is.null(W) || is.null(theta))
 			  return (varCHOLmerge(rbind(colMeans(dataL)),sig2,var.type,doInvert,Tnames))	
 			
-			# weighting matrix has to be inverted!		
+			# weighting matrix has to be inverted!
+			# but inverse of QI as variance of theta inverted is QI, thus W = I.
 			d <- try(exp(-.distX(Xs,rbind(theta),W)*0.5),silent=TRUE)
 			if(inherits(d,"try-error") || !is.numeric(d) || any(is.na(d))) {
 				msg <- paste0("Weighted distances calculation error: NAs values possibly produced.")
@@ -444,13 +455,9 @@ varCHOLmerge.numeric <- function(Xs, sig2=NULL, var.type="cholMean", doInvert=FA
 		res$var <- VTX + diag(sig2,n,n)
 
 		if(doInvert) {
-			res$inv <- try(do.call("solve",list(res$var)),silent=TRUE)
-			if(inherits(res$inv,"try-error")) {				
-				res$inv <- try(gsiInv(res$var),silent=TRUE)
-				if (inherits(res$inv,"try-error") || !is.numeric(res$inv) || any(is.na(res$inv))) {
-					return(.qleError(message="Variance matrix inversion failed: ",call=sys.call(),error=res))
-				}
-			}
+			res$inv <- try(do.call("gsiInv",list(res$var)),silent=TRUE)
+			if (inherits(res$inv,"try-error") || !is.numeric(res$inv) || any(is.na(res$inv)))
+			  return(.qleError(message="Variance matrix inversion failed: ",call=sys.call(),error=res))			
 		}
 	} else {
 		if(doInvert) {
@@ -482,22 +489,23 @@ varCHOLmerge.numeric <- function(Xs, sig2=NULL, var.type="cholMean", doInvert=FA
 #' @title Quasi-deviance computation
 #'
 #' @description
-#'  The function computes the quasi-deviance (QD) for parameters (called points) of the parameter
-#'  search space including the quasi-score vector and optionally its variance.   
+#'  The function computes the quasi-deviance (QD) at parameters (called points) of the parameter
+#'  search space including the computation of the quasi-score vector and optionally its variance matrix.   
 #'
-#' @param points		list or matrix of points where to compute the QD; a numeric vector is considered to be a point
+#' @param points		list or matrix of points where to compute the QD (a numeric vector is considered to be a (multidimensional) point)
 #' @param qsd		    object of class \code{\link{QLmodel}} 
 #' @param Sigma		    variance matrix estimate of statistics (see details)
 #' @param ...		    further arguments passed to \code{\link{covarTx}}
 #' @param cvm			list of cross-validation models (see \code{\link{prefitCV}})
-#' @param obs	 	    numeric vector of observed statistics, this overwrites `\code{qsd$obs}` if supplied
-#' @param inverted 		currently ignored
+#' @param obs	 	    numeric vector of observed statistics, this overwrites `\code{qsd$obs}`, if supplied
+#' @param inverted 		logical, \code{FALSE} (default), currently ignored
 #' @param check			logical, \code{TRUE} (default), whether to check input arguments
 #' @param value.only  	if \code{TRUE} only the values of the QD are returned
-#' @param cl			cluster object, \code{NULL} (default) (see \code{\link[parallel]{makeCluster}})
+#' @param na.rm 		logical, if \code{TRUE} (default) remove `Na`s from the result
+#' @param cl			cluster object, \code{NULL} (default), of class \code{MPIcluster}, \code{SOCKcluster}, \code{cluster}
 #' @param verbose   	logical, \code{TRUE} for intermediate output
 #'
-#' @return Numeric vector of QD values or a list as follows:
+#' @return Numeric vector of QD values, if values only, or a list with elements:
 #' \item{value}{ quasi-deviance value}
 #' \item{par}{ parameter estimate}
 #' \item{I}{ quasi-information matrix}
@@ -509,41 +517,42 @@ varCHOLmerge.numeric <- function(Xs, sig2=NULL, var.type="cholMean", doInvert=FA
 #' 
 #'  The matix `\code{Iobs}` is called the \eqn{\emph{observed quasi-information}} (see [2, Sec. 4.3]),
 #'  which, in our setting, can be calculated at least numerically as the Jacobian of the quasi-score vector.
-#'  Further, `\code{varS}` denotes the approximate variance-covariance matrix of the quasi-score vector given the observed
-#'  statistics and serves as a measure of accuracy (see [1] and the vignette, Sec. 3.2) of the approximation at some point.
+#'  Further, `\code{varS}` denotes the approximate variance-covariance matrix of the quasi-score given the observed
+#'  statistics and serves as a measure of estimation precision (see [1] and the vignette, Sec. 3.2).
 #'   
 #' @details The function calculates the QD (see [1]). It is the primary function criterion to be minimized
 #'   for estimating the unknown model parameter by \code{\link{qle}} and involves the computation of the quasi-score
 #'   and quasi-information matrix at a particular parameter. From a statistical point of view, the QD can be seen as
 #'   a generalization to the \emph{efficient score statistic} (see [3] and the vignette) and is used as a decision
-#'   rule in the estimation function \code{\link{qle}} in order to hypothesize about the true model parameter. A modified value of
-#'   the QD, using the inverse of the variance of the quasi-score vector as a weighting matrix, is stored in the result `\code{qval}`.
+#'   rule in the estimation function \code{\link{qle}} in order to hypothesise about the true model parameter. A modified value of
+#'   the QD, using the inverse of the variance of the quasi-score (as a weight matrix) is stored in the result `\code{qval}`.
 #'    
 #'   Quasi-deviance values which are relatively small (compared to the empirical quantiles of its approximate chi-squared
 #'   distribution) suggest a solution to the quasi-score equation and hence could identify the unknown model parameter
-#'   in some probabilistic sense. This can be further investigated by testing the hypothesis by function \code{\link{qleTest}}
-#'   whether the estimated model parameter is the true.
+#'   in some probabilistic sense. Estimated parameters including different observed statistics can be investigated by 
+#'   a MC goodness-of-fit test, see \code{\link{qleTest}}.
 #' 
 #'   Further, if we use a weighted variance average approximation of statistics (see \code{\link{covarTx}}),
-#'   then the QD value is calculated rather locally w.r.t. to an estimate `\code{theta}`. Note that, opposed to the MD,
-#'   the QD does not support a constant variance matrix. However, if supplied, then `\code{Sigma}` is used as a first estimate
-#'   and, if `\code{qsd$krig.type}`="\code{var}", prediction variances are also added (see also \code{\link{mahalDist}}).    
+#'   then the QD value is calculated rather locally w.r.t. to an estimated parameter `\code{theta}`. Note that, opposed to the MD,
+#'   a constant variance matrix is not applicable to the computation of the QD. However, if supplied, `\code{Sigma}` is used
+#'   as a first estimate and in case of `\code{qsd$krig.type}`="\code{var}" prediction variances of the involved statistics at
+#'   `\code{points}` are added as diagonal terms (see also \code{\link{mahalDist}}).    
 #' 
 #' 	 \subsection{Use of prediction variances}{ 
-#' 	 In order to not only account for the simulation error but additionally for the approximation error of the
+#' 	 In order to not only account for the simulation variance but additionally for the approximation error of the
 #'   quasi-score vector we include the prediction variances of the involved statistics either based on
-#'   cross-validation or kriging unless `\code{qsd$krig.type}` equals "\code{dual}". If `\code{cvm}` is not given, then
-#'   the prediction variances are obtained by kriging. Using prediction variances the error matrix `\code{varS}` of
-#'   the quasi-score vector is part of the return list and omitted otherwise. Besides the quasi-information matrix
-#'   also the observed quasi-information matrix (as a numerically derived Jacobian, given by `\code{Iobs}`, of the quasi-score vector)
-#'   is returned. A good match between those two matrices suggests a possible root (with some probablity) if the corresponding
-#'   QD value is relatively small. This can be further investigated by function \code{\link{checkMultRoot}}.
+#'   a cross-validation or kriging approach unless `\code{qsd$krig.type}` equals "\code{dual}". If `\code{cvm}` is not given, then
+#'   the prediction variances are obtained based on the kriging procedure applied to the statistics. Using prediction variances the
+#'   variance matrix `\code{varS}` of the quasi-score vector is part of the return list and omitted otherwise. Besides the quasi-information
+#'   matrix the observed quasi-information matrix (as a numerically derived Jacobian, given by `\code{Iobs}`, of the quasi-score vector)
+#'   is also returned. A good match between those two matrices suggests an approximate root if the corresponding
+#'   QD value is relatively small. This can be further investigated using the function \code{\link{checkMultRoot}}.
 #' 
-#'   Alternatively, also CV-based prediction variances (with additional covariance models given by `\code{cvm}`)
-#'   for each single statistic can be used to produce relatively robust estimation results but for the price of
+#'   Alternatively, also CV-based prediction variances (which involve additional covariance models given by `\code{cvm}` for each left out
+#'   sample point) for each single statistic can be used to produce relatively robust parameter estimation results but for the price of
 #'   much higher computational costs. In practice this might overcome the general tendency inherent to kriging to underestimate
-#'   the prediction variances of the sample means of the statistics and should be used if kriging the variance matrix of the statistics.
-#'   Further, CV is generally recommended in all situations where it is important to obtain a robust estimate of the unkown model parameter.
+#'   the prediction variances of the sample means of the statistics and should be used if one decides using kriging for the approximation of
+#'   the variance matrix. 
 #'   }
 #' 
 #' 
@@ -555,39 +564,41 @@ varCHOLmerge.numeric <- function(Xs, sig2=NULL, var.type="cholMean", doInvert=FA
 #' @rdname quasiDeviance
 #' @export
 quasiDeviance <- function(points, qsd, Sigma = NULL, ..., cvm = NULL, obs = NULL, 
-					 inverted = FALSE, check = TRUE, value.only=FALSE, 
+					 inverted = FALSE, check = TRUE, value.only=FALSE, na.rm = TRUE,
 					  cl = NULL, verbose=FALSE)
 {		
 	if(check)
-	 .checkArguments(qsd,Sigma=Sigma)
- 
+	 .checkArguments(qsd,Sigma=Sigma) 
 	if(!is.list(points))
-	 points <- .ROW2LIST(points)
- 	
+	 points <- .ROW2LIST(points) 	
  	X <- as.matrix(qsd$qldata[seq(attr(qsd$qldata,"xdim"))])
  	
 	# overwrite (observed) statistics	
 	if(!is.null(obs)) {
 		obs <- unlist(obs)
 		if(anyNA(obs) | any(!is.finite(obs)))
-			warning("`NA` or `Inf`values detected in `obs.")
-		if(!is.numeric(obs) || length(obs)!=length(qsd$covT))
+			warning("`NA` or `Inf` values detected in `obs`.")
+		if(!is.numeric(obs) || length(obs) != length(qsd$covT))
 			stop("`obs` must be a (named) `numeric` vector or list \n
 					of length equal to the given statistics in `qsd`.")
 		qsd$obs <- obs
 	}
-	# Unless Sigma is given always continuously update.
+	# Unless Sigma is given always continuously update variance matrix.
 	# If using W, theta or Sigma for average approximation, then
-	# at least update prediction variances at each point
+	# at least update prediction variances at each point.
 	tryCatch({
-		# use Sigma but add prediction variances
-		if(qsd$var.type != "kriging" && is.null(Sigma)){
-			# Sigma is inverted at C level
+		# use `Sigma` but add prediction variances
+		useSigma <- (qsd$var.type == "const")
+		if(qsd$var.type != "kriging" && !useSigma){
+			# Here: `Sigma` is inverted at C level
 			Sigma <- covarTx(qsd,...,cvm=cvm)[[1L]]$VTX
-		}			
-		qlopts <- list("varType"=qsd$var.type,			   
+		}
+		if(useSigma && is.null(Sigma))
+		 stop("`Sigma` cannot be NULL if used as a constant variance matrix.")
+		
+	    qlopts <- list("varType"=qsd$var.type,			   
 				       "useCV"=!is.null(cvm),
-					   "useSigma"=FALSE) # there is no constant Sigma 
+					   "useSigma"=FALSE) 			# there is no constant Sigma for quasi-deviance computation 
 
 		# somehow complicated but this is a load ballancing 
 		# (for a cluster) parallized version of quasiDeviance
@@ -611,22 +622,20 @@ quasiDeviance <- function(points, qsd, Sigma = NULL, ..., cvm = NULL, obs = NULL
 		}
 		
 		# check for NAs
-		has.na <- as.numeric(which(is.na(sapply(ret,"[",1))))	
-		if(length(has.na) == length(ret)){
-			stop("All quasi-deviance calculations produced `Na` values.")
-		}
-		if(length(has.na>0L)){		
-			warning("Removing `Na` values from results of quasi-deviance calculation.")
-			return(
-			structure(ret[-has.na],
-			  "hasNa"=has.na))
-        } else {
-		  return(ret)
+		if(na.rm){
+			has.na <- as.numeric(which(is.na(sapply(ret,"[",1))))	
+			if(length(has.na) == length(ret)){
+				stop("All quasi-deviance calculations produced `NA` values.")
+			}
+			if(length(has.na > 0L)){		
+				message("Removing `NA` values from results of quasi-deviance calculation.")
+				return( structure(ret[-has.na],  "hasNa"=has.na))
+			}
 		}
+		return(ret)
 
 	}, error = function(e) {
-		message(.makeMessage("Calculation of quasi-deviance failed: ",
-				   conditionMessage(e)))
+		message(.makeMessage("Calculation of quasi-deviance failed: ",conditionMessage(e)))
 	 	stop(e)  # re-throw error
 	})
 }
@@ -641,26 +650,27 @@ quasiDeviance <- function(points, qsd, Sigma = NULL, ..., cvm = NULL, obs = NULL
 #'  Compute the Mahalanobis distance (MD) based on the kriging models of statistics  	 
 #' 
 #' @param points	  either matrix or list of points or a vector of parameters (but then considered
-#' 					  as a single point)
+#' 					  as a single (multidimensional) point)
 #' @param qsd		  object of class \code{\link{QLmodel}} 
-#' @param Sigma		  either a constant variance matrix estimate or an pre-specified value 
+#' @param Sigma		  either a constant variance matrix estimate or an prespecified value 
 #' @param ...		  further arguments passed to \code{\link{covarTx}} for variance average approximation
 #' @param cvm		  list of fitted cross-validation models (see \code{\link{prefitCV}})
-#' @param obs         numeric vector of observed statistics (this overwrites `\code{qsd$obs}`)
+#' @param obs         numeric vector of observed statistics (overwrites `\code{qsd$obs}` if given)
 #' @param inverted    logical, \code{FALSE} (default), whether `\code{Sigma}` is already inverted when
-#' 					  used as constant variance matrix
+#' 					  used as constant variance matrix only
 #' @param check       logical, \code{TRUE} (default), whether to check all input arguments
 #' @param value.only  only return the value of the MD 
-#' @param cl		  cluster object, \code{NULL} (default), see \code{\link[parallel]{makeCluster}}
+#' @param na.rm   	  logical, if \code{TRUE} (default) remove `Na` values from the results
+#' @param cl		  cluster object, \code{NULL} (default), of class \code{MPIcluster}, \code{SOCKcluster}, \code{cluster}
 #' @param verbose     if \code{TRUE}, then print intermediate output
 #' 
 #' @return Either a vector of MD values or a list of lists, where each contains the following elements:
 #' \item{value}{ Mahalanobis distance value}
 #' \item{par}{ parameter estimate}
 #' \item{I}{ approximate variance matrix of the parameter estimate}
-#' \item{score}{ gradient of MD (for fixed `\code{Sigma}`)}
-#' \item{jac}{ Jacobian of sample average statistics}
-#' \item{varS}{ estimated variance of the gradient `\code{score}`}
+#' \item{score}{ gradient of MD (for constant `\code{Sigma}`)}
+#' \item{jac}{ Jacobian of sample mean values of statistics}
+#' \item{varS}{ estimated variance matrix of `\code{score}`}
 #' 
 #' and, if applicable, the following attributes:
 #' 
@@ -668,13 +678,13 @@ quasiDeviance <- function(points, qsd, Sigma = NULL, ..., cvm = NULL, obs = NULL
 #' \item{inverted}{ whether `\code{Sigma}` was inverted } 
 #'  
 #' @details	The function computes the Mahalanobis distance of the given statistics \eqn{T(X)\in R^p} with different options
-#'  how to approximate the variance matrix. The Mahalanobis distance can be used as an alternative criterion function for
-#'  estimating the unknown parameter during the main estimation function \code{\link{qle}}.
+#'  for the approximation type of the variance matrix. The Mahalanobis distance can be used as an alternative criterion function for
+#'  estimating the unknown model parameter during the main estimation function \code{\link{qle}}.
 #'  
 #'  There are several options how to estimate or choose the variance matrix of the statistics \eqn{\Sigma}.
 #'  First, in case of a given constant variance matrix estimate `\code{Sigma}`, the Mahalanobis distance reads
 #' 	\deqn{ (T(x)-E_{\theta}[T(X)])^t\Sigma^{-1}(T(x)-E_{\theta}[T(X)]) }
-#'  and `\code{Sigma}` is directly used.
+#'  and `\code{Sigma}` is used as given.
 #' 
 #'  As a second option, the variance matrix \eqn{\Sigma} can be estimated by the average approximation 
 #'  \deqn{\bar{V}=\frac{1}{n}\sum_{i=1}^n V_i  }
@@ -683,13 +693,13 @@ quasiDeviance <- function(points, qsd, Sigma = NULL, ..., cvm = NULL, obs = NULL
 #'  Unless `\code{qsd$var.type}` equals "\code{const}" additional prediction variances are added as diagonal terms to
 #'  account for the kriging approximation error of the statistics using kriging with calculation of kriging variances
 #'  if `\code{qsd$krig.type}` equal to "\code{var}". Otherwise no additional variances are added. A weighted version of
-#'  the these average approximation types is also available (see \code{\link{covarTx}}).
+#'  these average approximation types is also available (see \code{\link{covarTx}}).
 #'  
 #'  As a continuous version of variance approximation we use a kriging approach (see [1]). Then
 #'  \deqn{\Sigma(\theta) = Var_{\theta}(T(X))}
-#'  denotes the variance matrix which depends on the parameter \eqn{\theta\in R^q}, which corresponds to the
+#'  denotes the variance matrix which depends on the parameter \eqn{\theta\in R^q} and corresponds to the
 #'  formal function argument `\code{points}`. Each time a value of the criterion function is calculated for any parameter
-#'  `\code{point}` this matrix is estimated by the correpsonding kriging model defined in `\code{qsd$covL}` either with or
+#'  `\code{point}` the variance matrix is estimated by the correpsonding approach either with or
 #'  without using prediction variances as explained above. Note that in this case the argument `\code{Sigma}` is ignored.
 #' 
 #' @examples
@@ -705,7 +715,7 @@ quasiDeviance <- function(points, qsd, Sigma = NULL, ..., cvm = NULL, obs = NULL
 #' @rdname mahalDist
 #' @export
 mahalDist <- function(points, qsd, Sigma = NULL, ..., cvm = NULL, obs = NULL,
-		               inverted = FALSE, check = TRUE, value.only = FALSE,
+		               inverted = FALSE, check = TRUE, value.only = FALSE, na.rm = TRUE,
 					    cl = NULL, verbose = FALSE)
 {
 	  
@@ -732,21 +742,23 @@ mahalDist <- function(points, qsd, Sigma = NULL, ..., cvm = NULL, obs = NULL,
 	  #  2. Average approx. computed by W and at theta; or by kriging
 	  #  3. Sigma constant
 	  tryCatch({		
-		   useSigma <- (!is.null(Sigma) && qsd$var.type == "const")
-		   if(qsd$var.type != "kriging" && is.null(Sigma)){			   
+		   useSigma <- (qsd$var.type == "const")
+		   if(qsd$var.type != "kriging" && !useSigma){			   
 			   # Sigma is inverted at C level
 			   Sigma <- covarTx(qsd,...,cvm=cvm)[[1L]]$VTX			   
 		   } else if(useSigma && !inverted){
 				# Only for constant Sigma, which is used as is!
 				inverted <- TRUE
-				Sigma <- try(solve(Sigma),silent=TRUE)
+				Sigma <- try(gsiInv(Sigma),silent=TRUE)
 				if(inherits(Sigma,"try-error")) {
 				  msg <- paste0("Inversion of constant variance matrix failed.")
 				  message(msg)
 				  return(.qleError(message = msg,error=Sigma))
 			    }
-			}		  
-				
+			}
+			if(useSigma && is.null(Sigma))
+			 stop("`Sigma` cannot be NULL if used as a constant variance matrix.")
+							
 			qlopts <- list("varType"=qsd$var.type,
 						   "useCV"=!is.null(cvm),
 						   "useSigma"=useSigma)  			# use as constant Sigma 
@@ -774,20 +786,18 @@ mahalDist <- function(points, qsd, Sigma = NULL, ..., cvm = NULL, obs = NULL,
 				attr(Sigma,"inverted") <- inverted
 				attr(ret,"Sigma") <- Sigma			  
 	 		}
-			# check for NAs
-			has.na <- as.numeric(which(is.na(sapply(ret,"[",1))))	
-			if(length(has.na) == length(ret)){
-				stop("All  Mahalanobis distance calculations produced `Na` values.")
+			# check for NAs/NaNs
+			if(na.rm){
+				has.na <- as.numeric(which(is.na(sapply(ret,"[",1))))	
+				if(length(has.na) == length(ret)){
+					stop("All  Mahalanobis distance calculations produced `NA` values.")
+				}
+				if(length(has.na>0L)){
+					warning("Removed `NA` values from results of Mahalanobis distance calculation.")
+					return(	structure(ret[-has.na],	"hasNa"=has.na))
+			    }				
 			}
-			if(length(has.na>0L)){
-				warning("Removed `Na` values from results of Mahalanobis distance calculation.")
-				return(
-					structure(ret[-has.na],
-							"hasNa"=has.na))
-		    } else {
-				return(ret)
-			}
-					
+			return(ret)					
 							
 		}, error = function(e) {
 			 message(.makeMessage("Calculation of Mahalanobis-distance failed: ",
@@ -802,16 +812,17 @@ mahalDist <- function(points, qsd, Sigma = NULL, ..., cvm = NULL, obs = NULL,
 #'
 #' @title Multidimensional Latin Hypercube Sampling (LHS) generation
 #'
-#' @description The function generates or augments a multidimensional LHS in a hyperbox.
+#' @description The function generates or augments a multidimensional design in a hyperbox.
 #'
 #' @param N		    number of points to randomly select or augment an existing sample set
-#' @param lb	    lower bounds defining the (hyper)box
-#' @param ub 		upper bounds defining the (hyper)box 
+#' @param lb	    lower bounds defining the (hyper)box of the parameter search space
+#' @param ub 		upper bounds defining the (hyper)box of the parameter search space
 #' @param method    type of sampling, `\code{randomLHS}`, `\code{maximinLHS}` or `\code{augmentLHS}` 				    
 #' @param X 		optional, matrix of existing sample points, \code{NULL} (default), for augmentation only
 #' @param type 		either "\code{list}" or "\code{matrix}" as return type
 #'
-#' @return Return either a list or a matrix of sampled vectors or newly generated points if an existing sample set is augmented.
+#' @return Either return a list or matrix of sampled vectors or newly generated points if an existing sample set
+#'  was to be augmented.
 #'
 #' @rdname multiDimLHS
 #' @importFrom lhs randomLHS maximinLHS augmentLHS
